package com.sun.org.apache.bcel.internal.classfile;

import java.util.Objects;
import com.sun.org.apache.bcel.internal.generic.Type;
import java.io.IOException;
import java.io.DataInput;
import com.sun.org.apache.bcel.internal.util.BCELComparator;

public final class Method extends FieldOrMethod
{
    private static BCELComparator bcelComparator;
    private ParameterAnnotationEntry[] parameterAnnotationEntries;
    
    public Method() {
    }
    
    public Method(Method c) {
        super(c);
    }
    
    Method(DataInput file, ConstantPool constant_pool) throws IOException, ClassFormatException {
        super(file, constant_pool);
    }
    
    public Method(int access_flags, int name_index, int signature_index, Attribute[] attributes, ConstantPool constant_pool) {
        super(access_flags, name_index, signature_index, attributes, constant_pool);
    }
    
    @Override
    public void accept(Visitor v) {
        v.visitMethod(this);
    }
    
    public final Code getCode() {
        Attribute[] attributes = super.getAttributes();
        int length = attributes.length;
        int i = 0;
        while (i < length) {
            Attribute attribute = attributes[i];
            if ((attribute instanceof Code)) {
                return (Code)attribute;
            }
            ++i;
        }
        return null;
    }
    
    public final ExceptionTable getExceptionTable() {
        Attribute[] attributes = super.getAttributes();
        int length = attributes.length;
        int i = 0;
        while (i < length) {
            Attribute attribute = attributes[i];
            if ((attribute instanceof ExceptionTable)) {
                return (ExceptionTable)attribute;
            }
            ++i;
        }
        return null;
    }
    
    public final LocalVariableTable getLocalVariableTable() {
        Code code = this.getCode();
        if (code == null) {
            return null;
        }
        return code.getLocalVariableTable();
    }
    
    public final LineNumberTable getLineNumberTable() {
        Code code = this.getCode();
        if (code == null) {
            return null;
        }
        return code.getLineNumberTable();
    }
    
    @Override
    public final String toString() {
        String access = Utility.accessToString(super.getAccessFlags());
        ConstantUtf8 c = (ConstantUtf8)super.getConstantPool().getConstant(super.getSignatureIndex(), (byte)1);
        String signature = c.getBytes();
        c = (ConstantUtf8)super.getConstantPool().getConstant(super.getNameIndex(), (byte)1);
        String name = c.getBytes();
        signature = Utility.methodSignatureToString(signature, name, access, true, this.getLocalVariableTable());
        StringBuilder buf = new StringBuilder(signature);
        for (Attribute attribute : super.getAttributes()) {
            if (!(attribute instanceof Code) && !(attribute instanceof ExceptionTable)) {
                buf.append(" [").append(attribute).append("]");
            }
        }
        ExceptionTable e = this.getExceptionTable();
        if (e != null) {
            String str = e.toString();
            if (!str.isEmpty()) {
                buf.append("\n\t\tthrows ").append(str);
            }
        }
        return buf.toString();
    }
    
    public final Method copy(ConstantPool _constant_pool) {
        return (Method)this.copy_(_constant_pool);
    }
    
    public Type getReturnType() {
        return Type.getReturnType(this.getSignature());
    }
    
    public Type[] getArgumentTypes() {
        return Type.getArgumentTypes(this.getSignature());
    }
    
    public static BCELComparator getComparator() {
        return Method.bcelComparator;
    }
    
    public static void setComparator(BCELComparator comparator) {
        Method.bcelComparator = comparator;
    }
    
    @Override
    public boolean equals(Object obj) {
        return Method.bcelComparator.equals(this, obj);
    }
    
    @Override
    public int hashCode() {
        return Method.bcelComparator.hashCode(this);
    }
    
    public ParameterAnnotationEntry[] getParameterAnnotationEntries() {
        if (this.parameterAnnotationEntries == null) {
            this.parameterAnnotationEntries = ParameterAnnotationEntry.createParameterAnnotationEntries(this.getAttributes());
        }
        return this.parameterAnnotationEntries;
    }
    
    static {
        Method.bcelComparator = new BCELComparator() {
            @Override
            public boolean equals(Object o1, Object o2) {
                Method THIS = (Method)o1;
                Method THAT = (Method)o2;
                return Objects.equals(THIS.getName(), THAT.getName()) && Objects.equals(THIS.getSignature(), THAT.getSignature());
            }
            
            @Override
            public int hashCode(Object o) {
                Method THIS = (Method)o;
                return THIS.getSignature().hashCode() ^ THIS.getName().hashCode();
            }
        };
    }
}
